﻿using Microsoft.AspNetCore.Http.Extensions;
using NetBootServerCore.Components.Logging;
using System.Collections.Immutable;
using LogLevel = Microsoft.Extensions.Logging.LogLevel;

namespace NetBootServerCore.Components.Http
{
    public class HttpServer : NbsComponentBase
    {
        private readonly string[] _args;
        private readonly IReadOnlyCollection<IHttpRequestProcessor> _processors;
        private WebApplication _app;

        public HttpServer(string[] args, IEnumerable<IHttpRequestProcessor> processors) : base(new Logger("HTTP"))
        {
            _args = args;
            _processors = processors.ToImmutableArray();
        }

        protected override void OnStart()
        {
            var builder = WebApplication.CreateBuilder(_args);
            builder.Logging.SetMinimumLevel(LogLevel.None);
            _app = builder.Build();

            _app.Run(async context =>
            {
                context.Response.Headers.Server = nameof(NetBootServerCore);

                foreach (var processor in _processors) if (await processor.ProcessRequest(context)) return;

                context.Response.StatusCode = 404;
                Logger.LogWarn($"404 {context.Request.GetEncodedPathAndQuery()}");
            });

            _app.RunAsync();
        }

        protected override void OnStop()
        {
            _app.StopAsync();
        }
    }
}
