﻿namespace NetBootServerCore.Components.Logging
{
    public static class LogPool
    {
        //TODO: Data Logs

        public static event Action LogChanged;

        public static int RequestsHandled = 0;
        public static int InBytes = 0;
        public static int OutBytes = 0;

        private static readonly List<LogEntry> _logs = new();

        public static void Append(LogEntry entry)
        {
            lock (_logs)
            {
                _logs.Add(entry);
                LogChanged?.Invoke();
            }
        }

        public static void Clear()
        {
            lock (_logs)
            {
                _logs.Clear();
                LogChanged?.Invoke();
            }
        }

        public static LogEntry[] TakeLast(int num)
        {
            lock (_logs) return _logs.TakeLast(num).ToArray();
        }

        public static LogEntry[] Paging(int skip, int take)
        {
            lock (_logs) return _logs.Skip(skip).Take(take).ToArray();
        }

        public static int Count
        {
            get
            {
                lock (_logs) return _logs.Count;
            }
        }

        public static LogEntry GetEntryById(int id)
        {
            lock (_logs) return _logs.FirstOrDefault(p => p.Id == id);
        }
    }
}
