﻿using NetBootServerCore.Components.Logging;

namespace NetBootServerCore.Components
{
    public abstract class NbsComponentBase
    {
        protected Logger Logger { get; }

        protected NbsComponentBase(Logger logger)
        {
            Logger = logger;
        }

        public void Init()
        {
            Logger?.LogVerbose("Initialization");
            try
            {
                OnInit();
                Logger?.LogVerbose("Initialize OK");
            }
            catch (Exception e)
            {
                Logger?.LogError("Failure to Initialize", e.ToString());
            }
        }

        public void Start()
        {
            Logger?.LogVerbose("Starting");
            try
            {
                OnStart();
                Logger?.LogVerbose("Started");
            }
            catch (Exception e)
            {
                Logger?.LogError("Failure to Start", e.ToString());
            }
        }

        public void Stop()
        {
            Logger?.LogVerbose("Stopping");
            try
            {
                OnStop();
                Logger?.LogVerbose("Stopped");
            }
            catch (Exception e)
            {
                Logger?.LogError("Failure to Stop", e.ToString());
            }
        }

        protected virtual void OnInit()
        {
        }

        protected virtual void OnStart()
        {
        }

        protected virtual void OnStop()
        {
        }
    }
}
