﻿using NetBootServerCore.Components.Logging;
using SharpPcap.LibPcap;
using System.Net;

namespace NetBootServerCore.Components.NetBoot.Dhcp
{
    public class DhcpServer : NbsComponentBase
    {
        private const int Port = 67;
        private readonly IConfigurationSection _config;

        private IPEndPoint _listenOn;

        private IPAddress _poolStart;
        private int _poolSize;

        private IPAddress _subNet;
        private IPAddress _gateway;
        private IPAddress _dns;

        private IPAddress _broadcastAddress;

        private IPAddress _tftp;
        private string _fileName;
        private string _fileNameForIpxe;

        private static LibPcapLiveDevice _nic;

        public DhcpServer(IConfigurationSection config) : base(new Logger("DHCP")) => _config = config;

        protected override void OnInit()
        {
            _listenOn = new IPEndPoint(IPAddress.Parse(_config["ListenOn"]), 67);

            _poolStart = IPAddress.Parse(_config["PoolStart"]);
            _poolSize = int.Parse(_config["PoolSize"]);

            _subNet = IPAddress.Parse(_config["SubNet"]);
            _gateway = IPAddress.Parse(_config["Gateway"]);
            _dns = IPAddress.Parse(_config["Dns"]);

            _broadcastAddress = IPAddress.Parse(_config["BroadcastAddress"]);

            _tftp = IPAddress.Parse(_config["Tftp"]);
            _fileName = _config["FileName"];
            _fileNameForIpxe = _config["FileName_iPXE"];
        }

        protected override void OnStart()
        {
            _nic = LibPcapLiveDeviceList.Instance.FirstOrDefault(p => p.Addresses.Any(q => Equals(q.Addr.ipAddress, _listenOn.Address)));
        }

        protected override void OnStop()
        {
        }
    }
}
