﻿using Microsoft.Extensions.Options;
using System.Net;
using System.Text;
using static NetBootServerCore.Components.NetBoot.Dhcp.Protocol.Enums;

namespace NetBootServerCore.Components.NetBoot.Dhcp.Protocol
{
    public class DhcpPdu
    {
        public DhcpOpCode OpCode { get; set; }
        public DhcpHardwareType HardwareType { get; set; }
        public byte Hops { get; set; }
        public int TransactionId { get; set; }

        public short SecondsElapsed { get; set; }

        public DhcpBootpFlag BootpFlags { get; set; }
        public IPAddress ClientIpAddress { get; set; }

        public IPAddress YourIpAddress { get; set; }

        public IPAddress NextServerIpAddress { get; set; }

        public IPAddress RelayAgentIpAddress { get; set; }

        public byte[] ClientMacAddress { get; set; }

        public string ServerHostName { get; set; }
        public string BootFileName { get; set; }

        public int MagicCookie { get; set; } = 0x63825363;
        public Dictionary<DhcpOption, byte[]> Options { get; set; } = new Dictionary<DhcpOption, byte[]>();

        public DhcpMessageType MessageType
        {
            get => Options.MapValue(DhcpOption.DhcpMessageType, value => (DhcpMessageType)value[0]);

            set => Options[DhcpOption.DhcpMessageType] = new[] { (byte)value };
        }

        public IPAddress SubNetMask
        {
            get => Options.MapValue(DhcpOption.SubNetMask, value => new IPAddress(value));
            set => Options[DhcpOption.SubNetMask] = value.GetAddressBytes();
        }

        public IPAddress Router
        {
            get => Options.MapValue(DhcpOption.Router, value => new IPAddress(value));
            set => Options[DhcpOption.Router] = value.GetAddressBytes();
        }

        public IPAddress DnsServer
        {
            get => Options.MapValue(DhcpOption.DnsServer, value => new IPAddress(value));
            set => Options[DhcpOption.DnsServer] = value.GetAddressBytes();
        }

        public string TftpServer
        {
            get => Options.MapValue(DhcpOption.TftpServer, value => Encoding.ASCII.GetString(value));
            set => Options[DhcpOption.TftpServer] = Encoding.ASCII.GetBytes(value);
        }

        public IPAddress BroadcastAddress
        {
            get => Options.MapValue(DhcpOption.BroadcastAddress, value => new IPAddress(value));
            set => Options[DhcpOption.BroadcastAddress] = value.GetAddressBytes();
        }

        public IPAddress ServerIdentifier
        {
            get => Options.MapValue(DhcpOption.ServerIdentifier, value => new IPAddress(value));
            set => Options[DhcpOption.ServerIdentifier] = value.GetAddressBytes();
        }

        public IPAddress DhcpServerIdentifier
        {
            get => Options.MapValue(DhcpOption.DhcpServerIdentifier, value => new IPAddress(value));
            set => Options[DhcpOption.DhcpServerIdentifier] = value.GetAddressBytes();
        }

        public string UserClass
        {
            get => Options.MapValue(DhcpOption.UserClass, value => Encoding.ASCII.GetString(value))?.Trim().Trim('\0');
            set => Options[DhcpOption.UserClass] = Encoding.ASCII.GetBytes(value);
        }

        public string HostName => Options.MapValue(DhcpOption.HostName, v => Encoding.ASCII.GetString(v))?.Trim().Trim('\0');
        public string Vendor => Options.MapValue(DhcpOption.Vendor, v => Encoding.ASCII.GetString(v))?.Trim().Trim('\0');

        public TimeSpan LeaseTime
        {
            get => Options.MapValue(DhcpOption.LeaseTime, p => TimeSpan.FromSeconds(p.ToBeUint()));
            set => Options[DhcpOption.LeaseTime] = ((uint)value.TotalSeconds).ToBytes();
        }

        public TimeSpan RenewalTime
        {
            get => Options.MapValue(DhcpOption.RenewalTime, p => TimeSpan.FromSeconds(p.ToBeUint()));
            set => Options[DhcpOption.RenewalTime] = ((uint)value.TotalSeconds).ToBytes();
        }

        public TimeSpan RebindingTime
        {
            get => Options.MapValue(DhcpOption.RebindingTime, p => TimeSpan.FromSeconds(p.ToBeUint()));
            set => Options[DhcpOption.RebindingTime] = ((uint)value.TotalSeconds).ToBytes();
        }

        public byte[] ClientIdentifier
        {
            get => Options.MapValue(DhcpOption.ClientIdentifier, p => p);
            set => Options[DhcpOption.ClientIdentifier] = value;
        }

    }
}
