﻿using System.Text;

namespace NetBootServerCore.Components.NetBoot.Dhcp.Protocol;

public static class ExtensionMethods
{
    public static TR MapValue<TK, TV, TR>(this Dictionary<TK, TV> dic, TK key, Func<TV, TR> found)
    {
        return dic.TryGetValue(key, out var value)
            ? found(value)
            : default;
    }

    public static TR MapValue<TK, TV, TR>(this Dictionary<TK, TV> dic, TK key, Func<TV, TR> found, Func<TR> notFound)
    {
        return dic.TryGetValue(key, out var value)
            ? found(value)
            : notFound();
    }

    public static void FilterChars(this StringBuilder sb, HashSet<char> chars, char replace = '_')
    {
        for (var i = 0; i < sb.Length; i++)
        {
            if (chars.Contains(sb[i])) sb[i] = replace;
        }
    }

    public static uint ToBeUint(this byte[] buf)
    {
        return (uint)buf[0] << 24 |
               (uint)buf[1] << 16 |
               (uint)buf[2] << 8 |
               (uint)buf[3];
    }

    public static byte[] ToBytes(this uint value)
    {
        var buf = new byte[4];
        buf[0] = (byte)(value >> 24);
        buf[1] = (byte)(value >> 16);
        buf[2] = (byte)(value >> 8);
        buf[3] = (byte)(value);
        return buf;
    }
}